%module Xerces
%{
#include <stdio.h>
#include <strstream.h>
#include <string.h>
#include <sax/InputSource.hpp>
#include <sax/SAXException.hpp>
#include <sax/SAXParseException.hpp>
#include <sax/ErrorHandler.hpp>
#include <framework/URLInputSource.hpp>
#include <util/PlatformUtils.hpp>
#include <util/XMLString.hpp>
#include <parsers/DOMParser.hpp>
#include <dom/DOM.hpp>
#include <BinMemoryInputStream.hpp>
#include <BinFile2InputStream.hpp>
#include <framework/LocalFileInputSource.hpp>
#include <framework/MemBufInputSource.hpp>
#include <framework/StdInInputSource.hpp>
#include <framework/URLInputSource.hpp>
#include <MemoryInputSource.hpp>
#include <FileInputSource.hpp>
#include <perlErrorHandler.hpp>
%}

/**************/
/*            */
/*  TYPEMAPS  */
/*            */
/**************/

/*
 * FOR FUNCTIONS TAKING DOMString AS AN ARGUMENT -- NOW YOU CAN JUST SUPPLY A STRING
 * THIS TYPEMAP CONVERTS PERL-STRINGS TO DOMStrings AUTOMATICALLY
 */
%typemap(perl5, in) DOMString * (DOMString temp) {
  if (  SvPOK( $source )  ) {
    char *ptr = (char *)SvPV( $source, na );
    temp = DOMString(ptr);
    $target = &temp;
  } else if (SWIG_GetPtr($source,(void **) &$target,"DOMString")) {
    croak("Type error in argument 2 of $name, Expected perl-string or DOMString.");
    XSRETURN(1);
  }
}

/*
 * RETURN VALUE OF DOMString IS AUTOMATICALLY CONVERTED TO A PERL-STRING
 * (WITHOUT A CALL TO transcode IN THE PERL SCRIPT)
 */
%typemap(perl5, out) DOMString {
  $target = sv_newmortal();
  sv_setpv( (SV*)$target,(char *) $source->transcode() );
  delete $source;
  ++argvi;
}

/*
 * ARGUMENT TO CONSTRUCTOR OF LocalFileInputSource 
 */
%typemap(perl5, in) XMLCh * (XMLCh *temp) {
  if (  SvPOK( $source )  ) {
    char *ptr = (char *)SvPV( $source, na );
    $target = temp = XMLString::transcode(ptr);
  } else {
    croak("Type error in argument 2 of $name, Expected perl-string.");
    XSRETURN(1);
  }
}
/* KLUDGE! CLEAN UP WHAT temp ABOVE IS POINTING TO */
%typemap(perl5, freearg) XMLCh * {
  delete[] temp;
}

/*
 * FOR MemBufInputSource CONSTRUCTOR, ALWAYS ADOPT BUFFER (I.E. MAKE A COPY OF IT)
 * SINCE IT IS TAKEN FROM PERL, AND WHO KNOWS WHAT WILL HAPPEN TO IT
 * AFTER IT IS GIVEN TO THE CONSTRUCTOR
 */
%typemap(perl5, in) XMLByte *srcDocBytes {
  STRLEN length;
  if (  SvPOK( $source )  ) {
    XMLByte *xmlbytes = (XMLByte *)SvPV( $source, length );
    $target = new XMLByte[length];
    memcpy($target, xmlbytes, length);
    _arg1 = length;	// MAJOR KLUDGE -- SHOULDN'T HAVE TO PASS FROM PERL -- CALCULATED AUTOMATICALLY
  } else {
    croak("Type error in argument 2 of $name, Expected perl-string.");
    XSRETURN(1);
  }
}

/* PERL SHOULD IGNORE THIS ARGUMENT -- CALCULATE IN KLUDGE ABOVE FOR MemBufInputSource */
%typemap(perl5, ignore) unsigned int byteCount {
}

%typemap(perl5, ignore) bool adoptBuffer {
  $target = true;
}

/*
 * FOR perlErrorHandler MEMBER FUNCTIONS, SO PERL SCALAR DOESN'T GET WRAPPED BY SWIG
 */
%typemap(perl5, in) SV * {
  $target = $source;
}

/*
 * RETURN VALUE OF getMessage METHOD ON A saxException (WHAT A perlErrorHandler)
 * FUNCTION SUCH AS "warning" RECEIVES AS AN ARGUMENT
 */
%typemap(perl5, out) XMLCh * {
  char *cptr = XMLString::transcode($source);

  $target = sv_newmortal();
  sv_setpv( (SV*)$target, (char *)cptr );
  delete[] cptr;
  ++argvi;
}

/*******************/
/*                 */
/*  INCLUDE FILES  */
/*                 */
/*******************/
/* NECESSARY SUPPORT FILES (typedefs 'N THINGS) */
%include "Xerces_headers/util/XercesDefs.hpp"
%include "Xerces_headers/util/XMLURL.hpp"
%include "Xerces_headers/util/Compilers/GCCDefs.hpp"

/* NEEDED FOR INITIALIZATION AND TERMINATION */
%include "Xerces_headers/util/PlatformUtils.hpp"

/* FOR ERROR HANDLING (PARSER EXCEPTIONS IN PARTICULAR) */
%include "Xerces_headers/sax/SAXException.hpp"
%include "Xerces_headers/sax/SAXParseException.hpp"
%include "Xerces_headers/sax/ErrorHandler.hpp"
%include "New_classes_perl/perlErrorHandler.swig.hpp"

/* INCLUDE FILES IN "dom" DIRECTORY, ESSENTIALLY ALL OF THESE ARE NODE TYPES */
%include "Xerces_headers/dom/DOMString.hpp"
%include "Xerces_headers/dom/DOM_Attr.hpp"
%include "Xerces_headers/dom/DOM_CDATASection.hpp"
%include "Xerces_headers/dom/DOM_CharacterData.hpp"
%include "Xerces_headers/dom/DOM_Comment.hpp"
%include "Xerces_headers/dom/DOM_DOMImplementation.hpp"
%include "Xerces_headers/dom/DOM_Document.hpp"
%include "Xerces_headers/dom/DOM_DocumentFragment.hpp"
%include "Xerces_headers/dom/DOM_DocumentType.hpp"
%include "Xerces_headers/dom/DOM_Element.hpp"
%include "Xerces_headers/dom/DOM_Entity.hpp"
%include "Xerces_headers/dom/DOM_EntityReference.hpp"
%include "Xerces_headers/dom/DOM_NamedNodeMap.hpp"
%include "Xerces_headers/dom/DOM_Node.hpp"
%include "Xerces_headers/dom/DOM_NodeFilter.hpp"
%include "Xerces_headers/dom/DOM_NodeIterator.hpp"
%include "Xerces_headers/dom/DOM_NodeList.hpp"
%include "Xerces_headers/dom/DOM_Notation.hpp"
%include "Xerces_headers/dom/DOM_ProcessingInstruction.hpp"
%include "Xerces_headers/dom/DOM_Range.hpp"
%include "Xerces_headers/dom/DOM_Text.hpp"
%include "Xerces_headers/dom/DOM_TreeWalker.hpp"
%include "Xerces_headers/dom/DOM_XMLDecl.hpp"

/* INPUT SOURCES */
%include "Xerces_headers/sax/InputSource.hpp"
%include "New_classes_perl/MemoryInputSource.swig.hpp"
%include "New_classes_perl/FileInputSource.swig.hpp"
%include "Xerces_headers/framework/LocalFileInputSource.hpp"
%include "Xerces_headers/framework/MemBufInputSource.hpp"
%include "Xerces_headers/framework/StdInInputSource.hpp"
%include "Xerces_headers/framework/URLInputSource.hpp"

/* THE PARSER (PRETTY IMPORTANT) */
%include "Xerces_headers/parsers/DOMParser.hpp"

