#!/usr/bin/perl

my($progname) = $0 =~ m"\/([^/]*)";

%operator_symbols = (
  'addition'                  => '+',
  'addition_assignment'       => '+=',
  'subtraction'               => '-',
  'subtraction_assignment'    => '-=',
  'multiplication'            => '*',
  'multiplication_assignment' => '*=',
  'division'                  => '/',
  'division_assignment'       => '/=',
  'modulus'                   => '%',
  'modulus_assignment'        => '%=',

  'left_shift'                => '<<',
  'left_shift_assignment'     => '<<=',
  'right_shift'               => '>>',
  'right_shift_assignment'    => '>>=',

  'assignment'                => '=',

  'bitwise_and'               => '&',
  'bitwise_or'                => '|',
  'bitwise_xor'               => '^',

  'less_than'                 => '<',
  'less_than_or_equal_to'     => '<=',
  'greater_than'              => '>',
  'greater_than_or_equal_to'  => '>=',
  'equal_to'                  => '==',
  'not_equal_to'              => '!=',

  'increment'                 => '++',
  'decrement'                 => '--'  
);


unless (@ARGV) {
  my @operators;

  while(<>) {
    if (/\bpackage\b/) {
      if (@operators) {
        print "use Carp;\nuse overload\n";
        for $key (@operators) {
          my $operator_symbol = $operator_symbols{$key};
          print qq(\t'$operator_symbol' => ),
            qq(sub { carp "operator '$operator_symbol' assumed to be commutative." if \$_[2]; \$_[0]->$key(\$_[1]) },\n);
        }
        print qq(\t'fallback' => 1\n;\n\n);
        @operators = ();
      }
    }

    for $key (keys %operator_symbols) {
      push @operators, $key if /\b$key\b/;
    }

    s/(\$args\[([1-9]\d*)\])\s*=\s*tied\(%\{\$args\[\2\]\}\)/$& || $1/;

    if (/^(\s*)\$DOMString::OWNER/) {
    	print "    return \$result;\n}\n";
    	while(<>) { last if /^sub/; }
    }

    print;
  }

  exit 0;
}


my $num = 0;
++$num while -e "$progname.$num.tmp";

for $file (@ARGV) {
  next unless open FILE, $file;

  open TEMP, ">$progname.$num.tmp";

  while(<FILE>) {
    if (/\bpackage\b/) {
      if (@operators) {
        print TEMP "use Carp;\nuse overload\n";
        for $key (@operators) {
          my $operator_symbol = $operator_symbols{$key};
          print TEMP qq(\t'$operator_symbol' => ),
            qq(sub { carp "operator '$operator_symbol' assumed to be commutative." if \$_[2]; \$_[0]->$key(\$_[1]) },\n);
        }
        print TEMP qq(\t'fallback' => 1\n;\n\n);
        @operators = ();
      }
    }

    for $key (keys %operator_symbols) {
      push @operators, $key if /\b$key\b/;
    }

    s/(\$args\[([1-9]\d*)\])\s*=\s*tied\(%\{\$args\[\2\]\}\)/$& || $1/;

    if (/^(\s*)\$DOMString::OWNER/) {
    	print TEMP "    return \$result;\n}\n";
    	while(<FILE>) { last if /^sub/; }
    }

    print TEMP;
  }

  close FILE;

  print TEMP <<'TERMINUS';
package Xerces;
use RTTI ();

sub actual_cast {

  return undef unless RTTI::isa( ref($_[0]), 'DOM_Node' );
  return $_[0] if $_[0]->isNull;

  my $node_type = $_[0]->getNodeType;
  return RTTI::reinterpret_cast('DOM_Text', $_[0])                  if $node_type == $DOM_Node::TEXT_NODE;
  return RTTI::reinterpret_cast('DOM_ProcessingInstruction', $_[0]) if $node_type == $DOM_Node::PROCESSING_INSTRUCTION_NODE;
  return RTTI::reinterpret_cast('DOM_Document', $_[0])              if $node_type == $DOM_Node::DOCUMENT_NODE;
  return RTTI::reinterpret_cast('DOM_Element', $_[0])               if $node_type == $DOM_Node::ELEMENT_NODE;
  return RTTI::reinterpret_cast('DOM_EntityReference', $_[0])       if $node_type == $DOM_Node::ENTITY_REFERENCE_NODE;
  return RTTI::reinterpret_cast('DOM_CDATASection', $_[0])          if $node_type == $DOM_Node::CDATA_SECTION_NODE;
  return RTTI::reinterpret_cast('DOM_Comment', $_[0])               if $node_type == $DOM_Node::COMMENT_NODE;
  return RTTI::reinterpret_cast('DOM_DocumentType', $_[0])          if $node_type == $DOM_Node::DOCUMENT_TYPE_NODE;
  return RTTI::reinterpret_cast('DOM_Entity', $_[0])                if $node_type == $DOM_Node::ENTITY_NODE;
  return RTTI::reinterpret_cast('DOM_XMLDecl', $_[0])               if $node_type == $DOM_Node::XML_DECL_NODE;

  return $_[0];
}


sub quote_content {
  my $node_value = $_[0];

  $node_value =~ s/&/&amp;/g;
  $node_value =~ s/</&lt;/g;
  $node_value =~ s/>/&gt;/g;
  $node_value =~ s/"/&quot;/g;
  $node_value =~ s/'/&apos;/g;

  return $node_value;
}

{
  my $in_element = 0;
  my $output;
  my @output_array;

  $output_array[$DOM_Node::TEXT_NODE] = sub {
    $output .= quote_content $_[0]->getNodeValue;
  };

  $output_array[$DOM_Node::PROCESSING_INSTRUCTION_NODE] = sub {
    $output .= '<?' . $_[0]->getNodeName;
    if ( length(my $str = $_[0]->getNodeValue) ) { $output .= " $str"; }
    $output .= '?>';
  };

  $output_array[$DOM_Node::DOCUMENT_NODE] = sub {
    for(my $child = $_[0]->getFirstChild->actual_cast ;
     !$child->isNull ;
     $child = $child->getNextSibling->actual_cast) {
      &{$output_array[$child->getNodeType]}($child);
    }
  };

  $output_array[$DOM_Node::ELEMENT_NODE] = sub {
    ++$in_element;
    ELEMENT: {
      my $node_name = $_[0]->getNodeName;
      $output .= "<$node_name";

      my $attributes = $_[0]->getAttributes;
      my $attribute_count = $attributes->getLength;

      for(my $ix = 0 ; $ix < $attribute_count ; ++$ix) {
        $attribute = $attributes->item($ix);
        $output .= ' ' . $attribute->getNodeName . '="' . quote_content($attribute->getNodeValue) . '"';
      }

      my $child = $_[0]->getFirstChild->actual_cast;
      ($output .= '/>'), last ELEMENT if $child->isNull;

      $output .= '>';
      until($child->isNull) {
        &{$output_array[$child->getNodeType]}($child);
        $child = $child->getNextSibling->actual_cast;
      }
      $output .= "</$node_name>";
    }
    --$in_element;
    $output .= "\n" unless $in_element;
  };

  $output_array[$DOM_Node::ENTITY_REFERENCE_NODE] = sub {
    for(my $child = $_[0]->getFirstChild->actual_cast ;
     !$child->isNull ;
     $child = $child->getNextSibling->actual_cast) {
      &{$output_array[$child->getNodeType]}($child);
    }
  };

  $output_array[$DOM_Node::CDATA_SECTION_NODE] = sub {
    $output .= '<![CDATA[' . $_[0]->getNodeValue . ']]>';
  };

  $output_array[$DOM_Node::COMMENT_NODE] = sub {
    $output .= '<!--' . $_[0]->getNodeValue . '-->';
    $output .= "\n" unless $in_element;
  };

  $output_array[$DOM_Node::DOCUMENT_TYPE_NODE] = sub {
    $output .= '<!DOCTYPE ' . $_[0]->getNodeName;

    my $id;
    if ($id = $_[0]->getPublicId) {
      $output .= qq[ PUBLIC "$id"];
      if ($id = $_[0]->getSystemId) {
        $output .= qq[ "$id"];
      }
    } elsif ($id = $_[0]->getSystemId) {
      $output .= qq[ SYSTEM "$id"];
    }

    if ($id = $_[0]->getInternalSubset) {
      $output .= " [$id]";
    }

    $output .= ">\n"
  };

  $output_array[$DOM_Node::ENTITY_NODE] = sub {
    $output .= '<!ENTITY ' . $_[0]->getNodeName;

    my $id;
    if ($id = $_[0]->getPublicId) { $output .= qq[ PUBLIC "$id"]; }
    if ($id = $_[0]->getSystemId) { $output .= qq[ SYSTEM "$id"]; }
    if ($id = $_[0]->getNotationName) { $output .= qq[ NDATA "$id"]; }

    $output .= '>';
  };

  $output_array[$DOM_Node::XML_DECL_NODE] = sub {
    $output .= '<?xml version="' . $_[0]->getVersion . '" encoding="' . $_[0]->getEncoding;
    if (my $id = $_[0]->getStandalone) { $output .= qq[" standalone="$id]; }
    $output .= "\"?>\n\n";
  };

  sub serialize {
    $output = '';
    &{$output_array[ $_[0]->getNodeType ]}($_[0]);
    $output;
  }
}

XMLPlatformUtils::Initialize();

1;
TERMINUS

  close TEMP;

  rename "$progname.$num.tmp", $file;
}


